/*
 * This file is part of jPlot
 *
 * jPlot is a Java version of Tim Shepard's xplot which is distributed 
 * under the M.I.T. license (as below).
 *
 * Tim Shepard's original xplot is an amazing piece of software. It was
 * designed to be extraordinarily fast and efficient and as such, it has
 * only a very thin user interface. Those design goals, however, make
 * xplot extremely difficult to port to new architectures and also make it
 * difficult to use for the casual user. jPlot was designed to address
 * these limitations while keeping as much of the original efficiency as
 * possible. We thank Tim Shepard for his hard work on xplot and hope that
 * jPlot can live up to his high standards.
 *
 * jPlot was developed by Avinash Lakhiani at the Internetworking Research
 * Lab (IRG), Ohio University, as part of Dr. Shawn Ostermann's tcptrace
 * project. The main goal behind the development of jPlot was to make the 
 * plotting tool used with tcptrace more portable. As a result, jPlot
 * retains many of the original algorithms implemented in xplot, and as per
 * the terms and conditions of the M.I.T. license, continue to remain under
 * that license. The M.I.T license is as follows:
 *
 * --- M.I.T License Begin ---
 * 
 * Copyright 1992,1993 by the Massachusetts Institute of Technology.
 *                   All rights reserved.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS", AND M.I.T. MAKES NO REPRESENTATIONS
 * OR WARRANTIES, EXPRESS OR IMPLIED.  By way of example, but not
 * limitation, M.I.T. MAKES NO REPRESENTATIONS OR WARRANTIES OF
 * MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR PURPOSE OR THAT THE USE
 * OF THE LICENSED SOFTWARE OR DOCUMENTATION WILL NOT INFRINGE ANY THIRD
 * PARTY PATENTS, COPYRIGHTS, TRADEMARKS OR OTHER RIGHTS.
 *
 * The name of the Massachusetts Institute of Technology or M.I.T. may
 * NOT be used in advertising or publicity pertaining to distribution of
 * the software.  Title to copyright in this software and any associated
 * documentation shall at all times remain with M.I.T., and USER agrees
 * to preserve same.
 * 
 * --- M.I.T. License End ---
 * 
 * jPlot also has a lot of added features that are not part of the original
 * xplot code. Those sections of code are distributed under the following
 * BSD License:
 *
 * --- BSD License Begin ---
 *
 * Copyright (c) 2002, 2003 by Avinash Lakhiani
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *	
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer. 
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution. 
 *   - Neither the name of Ohio University nor the names of its contributors
 *     may be used to endorse or promote products derived from this software
 *     without specific prior written permission. 
 *	   
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * --- BSD License End ---
 * 
 * You should have received a file named "DISTINCTION" along with this
 * distribution, that clearly identifies all the algorithms that were
 * retained from the original xplot code, and remain under the M.I.T.
 * license. The rest of the code was written for jPlot, and falls under
 * the BSD license.
 * 
 * jPlot is free software; you can redistribute it and/or modify it under the
 * terms and conditions of both the above licenses.
 *
 * jPlot is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the above licenses for more details.
 *
 * Author:  Avinash Lakhiani
 *          Internetworking Research Group
 *          Ohio University
 *          Athens, OH
 *          avinash.lakhiani@ohiou.edu
 *          http://www.tcptrace.org/jPlot
 *
 */

/*
 * $Id: AboutBox.java,v 1.2 2002/12/04 01:58:52 alakhian Exp $
 */ 

/* Import Packages */
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

/* Class       - AboutBox
 * Description - this class implements the about dialog that displays distribution information
 * Extends     - JDialog
 * Implements  - ActionListener
 */
public class AboutBox extends JDialog implements ActionListener
{
   /* Member Variables */
   JPanel       panel1        = new JPanel();
   JPanel       panel2        = new JPanel();
   JPanel       insetsPanel1  = new JPanel();
   JPanel       insetsPanel2  = new JPanel();
   JPanel       insetsPanel3  = new JPanel();
   JButton      button1       = new JButton();
   JLabel       imageLabel    = new JLabel();
   JLabel       label1        = new JLabel();
   JLabel       label2        = new JLabel();
   JLabel       label3        = new JLabel();
   JLabel       label4        = new JLabel();
   JLabel       label5        = new JLabel();
   JLabel       label6        = new JLabel();
   BorderLayout borderLayout1 = new BorderLayout();
   BorderLayout borderLayout2 = new BorderLayout();
   FlowLayout   flowLayout1   = new FlowLayout();
   GridLayout   gridLayout1   = new GridLayout();
   
   public AboutBox(Frame parent)
     {
	super(parent);
	enableEvents(AWTEvent.WINDOW_EVENT_MASK);
	try
	  {
	     jbInit();
	  }
	catch(Exception e)
	  {
	     e.printStackTrace();
	  }
	pack();
     }
   
   /* Component initialization */
   private void jbInit() throws Exception
     {
	this.setTitle("About");
	panel1.setLayout(borderLayout1);
	panel2.setLayout(borderLayout2);
	insetsPanel1.setLayout(flowLayout1);
	insetsPanel2.setLayout(flowLayout1);
	insetsPanel2.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
	gridLayout1.setRows(6);
	gridLayout1.setColumns(1);
	label1.setText(Constant.PROGRAM_NAME_SHORT);
	label2.setText(" ");
	label3.setText("Version: " + Constant.VERSION_MAJOR + "." + Constant.VERSION_MINOR + "." + Constant.VERSION_BUGFIX);
	label4.setText("Author: "  + Constant.AUTHOR);
	label5.setText(" ");
	label6.setText(Constant.COPYRIGHT);
	insetsPanel3.setLayout(gridLayout1);
	insetsPanel3.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 30));
	button1.setText("Ok");
	button1.addActionListener(this);
	insetsPanel2.add(imageLabel, null);
	panel2.add(insetsPanel2, BorderLayout.WEST);
	this.getContentPane().add(panel1, null);
	insetsPanel3.add(label1, null);
	insetsPanel3.add(label2, null);
	insetsPanel3.add(label3, null);
	insetsPanel3.add(label4, null);
	insetsPanel3.add(label5, null);
	insetsPanel3.add(label6, null);	
	panel2.add(insetsPanel3, BorderLayout.CENTER);
	insetsPanel1.add(button1, null);
	panel1.add(insetsPanel1, BorderLayout.SOUTH);
	panel1.add(panel2, BorderLayout.NORTH);
     }
   
   /* Overridden so we can exit when window is closed */
   protected void processWindowEvent(WindowEvent e)
     {
	if (e.getID() == WindowEvent.WINDOW_CLOSING)
	  {
	     cancel();
	  }
	super.processWindowEvent(e);
     }
   
   /* Close the dialog */
   void cancel()
     {
	dispose();
     }
   
   /* Close the dialog on a button event */
   public void actionPerformed(ActionEvent e)
     {
	if (e.getSource() == button1)
	  {
	     cancel();
	  }
     }
}
